@@ -2,6 +2,7 @@
 from the actual STOMP protocol
 """
 
+import datetime
 import errno
 import math
 import random
@@ -12,34 +13,49 @@ from io import BytesIO
 from time import monotonic
 
 try:
-    import ssl
-    from ssl import SSLError
-
-    DEFAULT_SSL_VERSION = ssl.PROTOCOL_TLSv1
-except (ImportError, AttributeError):
-    ssl = None
-
-
-    class SSLError(object):
-        pass
-
-
-    DEFAULT_SSL_VERSION = None
-
-try:
-    from socket import SOL_SOCKET, SO_KEEPALIVE
-    from socket import SOL_TCP, TCP_KEEPIDLE, TCP_KEEPINTVL, TCP_KEEPCNT
-
+    from socket import SOL_SOCKET, SO_KEEPALIVE, SOL_TCP, TCP_KEEPIDLE, TCP_KEEPINTVL, TCP_KEEPCNT
     LINUX_KEEPALIVE_AVAIL = True
 except ImportError:
     LINUX_KEEPALIVE_AVAIL = False
 
+try:
+    from socket import IPPROTO_TCP
+    MAC_KEEPALIVE_AVAIL = True
+except ImportError:
+    MAC_KEEPALIVE_AVAIL = False
+
+try:
+    import ssl
+    from ssl import SSLError
+    DEFAULT_SSL_VERSION = ssl.PROTOCOL_TLS
+    from cryptography import x509
+except (ImportError, AttributeError):
+    ssl = None
+    class SSLError(object):
+        pass
+    DEFAULT_SSL_VERSION = None
+
+
 import stomp.exception as exception
 import stomp.listener
 from stomp.utils import *
 from stomp import logging
 
 
+def check_ssl_certificate(host_and_port):
+    '''
+    Check the expiry date of the certificate presented by the host/port.
+    '''
+    if ssl:
+        host, port = host_and_port
+        cert = ssl.get_server_certificate((host, port))
+        x509_cert = x509.load_pem_x509_certificate(cert.encode())
+        valid_cert = x509_cert.not_valid_after > datetime.datetime.now()
+        if not valid_cert:
+            logging.info("SSL certificate for %s:%s expired on %s", host, port, dt)
+        assert valid_cert
+
+
 class BaseTransport(stomp.listener.Publisher):
     """
     Base class for transport classes providing support for listeners, threading overrides,
@@ -358,13 +374,13 @@ class BaseTransport(stomp.listener.Publisher):
         finally:
             with self.__receiver_thread_exit_condition:
                 self.__receiver_thread_exited = True
-                self.__receiver_thread_exit_condition.notifyAll()
+                self.__receiver_thread_exit_condition.notify_all()
             logging.info("Receiver loop ended")
             self.notify("receiver_loop_completed")
             if notify_disconnected and not self.notified_on_disconnect:
                 self.notify("disconnected")
             with self.__connect_wait_condition:
-                self.__connect_wait_condition.notifyAll()
+                self.__connect_wait_condition.notify_all()
             self.notified_on_disconnect = False
 
     def __read(self):
@@ -479,19 +495,15 @@ class Transport(BaseTransport):
         an extra 0%-10% (randomly determined) of the delay
         calculated using the previous three parameters.
     :param int reconnect_attempts_max: maximum attempts to reconnect (Can also be used for infinite attempts : `-1`)
-    :param bool use_ssl: deprecated, see :py:meth:`set_ssl`
-    :param ssl_cert_file: deprecated, see :py:meth:`set_ssl`
-    :param ssl_key_file: deprecated, see :py:meth:`set_ssl`
-    :param ssl_ca_certs: deprecated, see :py:meth:`set_ssl`
-    :param ssl_cert_validator: deprecated, see :py:meth:`set_ssl`
-    :param ssl_version: deprecated, see :py:meth:`set_ssl`
     :param timeout: the timeout value to use when connecting the stomp socket
     :param keepalive: some operating systems support sending the occasional heart
         beat packets to detect when a connection fails.  This
         parameter can either be set set to a boolean to turn on the
         default keepalive options for your OS, or as a tuple of
         values, which also enables keepalive packets, but specifies
-        options specific to your OS implementation
+        options specific to your OS implementation.
+        For linux, supply ("linux", ka_idle, ka_intvl, ka_cnt)
+        For macos, supply ("mac", ka_intvl)
     :param str vhost: specify a virtual hostname to provide in the 'host' header of the connection
     :param int recv_bytes: the number of bytes to use when calling recv
     """
@@ -505,12 +517,6 @@ class Transport(BaseTransport):
                  reconnect_sleep_jitter=0.1,
                  reconnect_sleep_max=60.0,
                  reconnect_attempts_max=3,
-                 use_ssl=False,
-                 ssl_key_file=None,
-                 ssl_cert_file=None,
-                 ssl_ca_certs=None,
-                 ssl_cert_validator=None,
-                 ssl_version=None,
                  timeout=None,
                  keepalive=None,
                  vhost=None,
@@ -570,15 +576,6 @@ class Transport(BaseTransport):
 
         # setup SSL
         self.__ssl_params = {}
-        if use_ssl:
-            warnings.warn("Deprecated: use set_ssl instead", DeprecationWarning)
-            self.set_ssl(host_and_ports,
-                         ssl_key_file,
-                         ssl_cert_file,
-                         ssl_ca_certs,
-                         ssl_cert_validator,
-                         ssl_version)
-
         self.__keepalive = keepalive
         self.vhost = vhost
         self.__recv_bytes = recv_bytes
@@ -708,6 +705,10 @@ class Transport(BaseTransport):
                 ka_sig = "linux"
                 ka_args = None
                 logging.info("keepalive: autodetected linux-style support")
+            elif MAC_KEEPALIVE_AVAIL:
+                ka_sig = "mac"
+                ka_args = None
+                logging.info("keepalive: autodetected mac-style support")
             else:
                 logging.error("keepalive: unable to detect any implementation, DISABLED!")
                 return
@@ -717,11 +718,19 @@ class Transport(BaseTransport):
             if ka_args is None:
                 logging.info("keepalive: using system defaults")
                 ka_args = (None, None, None)
-            lka_idle, lka_intvl, lka_cnt = ka_args
+            ka_idle, ka_intvl, ka_cnt = ka_args
             if try_setsockopt(self.socket, "enable", SOL_SOCKET, SO_KEEPALIVE, 1):
-                try_setsockopt(self.socket, "idle time", SOL_TCP, TCP_KEEPIDLE, lka_idle)
-                try_setsockopt(self.socket, "interval", SOL_TCP, TCP_KEEPINTVL, lka_intvl)
-                try_setsockopt(self.socket, "count", SOL_TCP, TCP_KEEPCNT, lka_cnt)
+                try_setsockopt(self.socket, "idle time", SOL_TCP, TCP_KEEPIDLE, ka_idle)
+                try_setsockopt(self.socket, "interval", SOL_TCP, TCP_KEEPINTVL, ka_intvl)
+                try_setsockopt(self.socket, "count", SOL_TCP, TCP_KEEPCNT, ka_cnt)
+        elif ka_sig == "mac":
+            logging.info("keepalive: activating mac-style support")
+            if ka_args is None:
+                logging.info("keepalive: using system defaults")
+                ka_args = (3,)
+            ka_intvl = ka_args
+            if try_setsockopt(self.socket, "enable", SOL_SOCKET, SO_KEEPALIVE, 1):
+                try_setsockopt(self.socket, socket.IPPROTO_TCP, 0x10, ka_intvl)
         else:
             logging.error("keepalive: implementation %r not recognized or not supported", ka_sig)
 
@@ -747,15 +756,14 @@ class Transport(BaseTransport):
                     need_ssl = self.__need_ssl(host_and_port)
 
                     if need_ssl:  # wrap socket
+                        check_ssl_certificate(host_and_port)
                         ssl_params = self.get_ssl(host_and_port)
+                        tls_context = ssl.SSLContext(DEFAULT_SSL_VERSION)
                         if ssl_params["ca_certs"]:
                             cert_validation = ssl.CERT_REQUIRED
+                            tls_context.load_verify_locations(ssl_params["ca_certs"])
                         else:
                             cert_validation = ssl.CERT_NONE
-                        try:
-                            tls_context = ssl.create_default_context(cafile=ssl_params["ca_certs"])
-                        except AttributeError:
-                            tls_context = None
                         if tls_context:
                             # Wrap the socket for TLS
                             certfile = ssl_params["cert_file"]
@@ -768,9 +776,11 @@ class Transport(BaseTransport):
                             if cert_validation is None or cert_validation == ssl.CERT_NONE:
                                 tls_context.check_hostname = False
                             tls_context.verify_mode = cert_validation
+                            logging.info("Wrapping SSL socket")
                             self.socket = tls_context.wrap_socket(self.socket, server_hostname=host_and_port[0])
                         else:
                             # Old-style wrap_socket where we don't have a modern SSLContext (so no SNI)
+                            logging.info("Wrapping SSL socket (old style)")
                             self.socket = ssl.wrap_socket(
                                 self.socket,
                                 keyfile=ssl_params["key_file"],
@@ -796,7 +806,7 @@ class Transport(BaseTransport):
                     self.current_host_and_port = host_and_port
                     logging.info("Established connection to host %s, port %s", host_and_port[0], host_and_port[1])
                     break
-                except socket.error:
+                except (OSError, AssertionError):
                     self.socket = None
                     connect_count += 1
                     logging.warning("Could not connect to host %s, port %s", host_and_port[0], host_and_port[1],
